"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendTransaction = exports.createTxAndSend = exports.setupTxCallbacks = exports.makeCombinedCallback = exports.createQueryInvalidation = exports.mergeQueryConfig = exports.queryWrapper = void 0;
const utils_1 = require("./utils");
function queryWrapper(func, getAdditionalData) {
    return async ({ queryKey }) => {
        const data = queryKey[1];
        const additionalData = await getAdditionalData();
        return func({ ...additionalData, data });
    };
}
exports.queryWrapper = queryWrapper;
function mergeQueryConfig(config, defaultConfig) {
    return {
        ...defaultConfig,
        ...config,
        enabled: (defaultConfig?.enabled ?? true) && (config?.enabled ?? true),
    };
}
exports.mergeQueryConfig = mergeQueryConfig;
function createQueryInvalidation(client, key) {
    return (data = null, exact = false) => {
        client.invalidateQueries({
            queryKey: [key, data],
            exact,
        });
    };
}
exports.createQueryInvalidation = createQueryInvalidation;
function makeCombinedCallback(defaultConfig, config, attr) {
    return (...data) => {
        defaultConfig && defaultConfig[attr] && defaultConfig[attr](...data);
        config && config[attr] && config[attr](...data);
    };
}
exports.makeCombinedCallback = makeCombinedCallback;
const DEFAULT_TX_CALLBACKS = {
    onBroadcast: ({ summary }) => console.info(`Broadcasting ${summary}...`),
    onError: ({ error }) => console.error(error),
    onSuccess: ({ summary }) => console.log(`Success submit ${summary}...`),
};
let txCallbacks = DEFAULT_TX_CALLBACKS;
const setupTxCallbacks = (callbacks) => {
    txCallbacks = { ...DEFAULT_TX_CALLBACKS, ...callbacks };
};
exports.setupTxCallbacks = setupTxCallbacks;
async function createTxAndSend(transactionGenerator, param, additionalParams, txConfig, config, defaultConfig) {
    const { tx, summary } = await transactionGenerator(param, additionalParams);
    return sendTransaction({
        tx,
        wallet: txConfig.wallet,
        params: param,
        networkRpc: txConfig.networkRpc,
        summary,
    }, config, defaultConfig);
}
exports.createTxAndSend = createTxAndSend;
function sendTransaction(txInfo, config, defaultConfig) {
    const { networkRpc, params, summary, tx, wallet: { address, signer }, } = txInfo;
    return new Promise(async (resolve, reject) => {
        try {
            const unsub = await tx.signAndSend(address, {
                signer,
            }, async (result) => {
                resolve(result.txHash.toString());
                if (result.status.isBroadcast) {
                    txCallbacks.onBroadcast({
                        summary,
                        params: params,
                        address,
                    });
                }
                else if (result.status.isInBlock) {
                    const blockHash = (result.status.toJSON() ?? {}).inBlock;
                    let explorerLink;
                    if (networkRpc) {
                        explorerLink = (0, utils_1.getBlockExplorerBlockInfoLink)(networkRpc, blockHash);
                    }
                    if (result.isError ||
                        result.dispatchError ||
                        result.internalError) {
                        txCallbacks.onError({
                            error: result.dispatchError?.toString(),
                            summary,
                            address,
                            params,
                            explorerLink,
                        });
                    }
                    else {
                        const onTxSuccess = makeCombinedCallback(defaultConfig, config, 'onTxSuccess');
                        onTxSuccess(params, address);
                        txCallbacks.onSuccess({ explorerLink, summary, address, params });
                    }
                    unsub();
                }
            });
        }
        catch (e) {
            txCallbacks.onError(e.message);
            reject(e);
        }
    });
}
exports.sendTransaction = sendTransaction;
