"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSubsocialApi = exports.setSubsocialConfig = exports.getConnectionConfig = void 0;
const api_1 = require("@subsocial/api");
const DEFAULT_STAGING_CONFIG = {
    substrateUrl: 'wss://rco-para.subsocial.network',
    ipfsNodeUrl: 'https://staging-ipfs.subsocial.network',
    offchainUrl: 'https://staging-api.subsocial.network',
    useServer: {
        httpRequestMethod: 'get',
    },
};
const DEFAULT_PROD_CONFIG = {
    substrateUrl: 'wss://para.f3joule.space',
    offchainUrl: 'https://app.subsocial.network/offchain',
    ipfsNodeUrl: 'https://ipfs.subsocial.network',
};
let subsocialApi = null;
const presets = {
    staging: DEFAULT_STAGING_CONFIG,
    prod: DEFAULT_PROD_CONFIG,
};
const DEFAULT_CONFIG_PRESET = 'staging';
let config = presets[DEFAULT_CONFIG_PRESET];
function getConnectionConfig() {
    return config;
}
exports.getConnectionConfig = getConnectionConfig;
const setSubsocialConfig = (preset, customConfig) => {
    config = { ...presets[preset], ...customConfig };
};
exports.setSubsocialConfig = setSubsocialConfig;
const getSubsocialApi = async () => {
    if (subsocialApi)
        return subsocialApi;
    const api = connectToSubsocialApi(config);
    subsocialApi = api;
    return subsocialApi;
};
exports.getSubsocialApi = getSubsocialApi;
async function connectToSubsocialApi(config) {
    const substrateApi = await (0, api_1.getSubstrateApi)(config.substrateUrl);
    return new api_1.SubsocialApi({ substrateApi, ...config });
}
