import { useMutation, useQuery } from '@tanstack/react-query';
import { createTxAndSend, makeCombinedCallback, mergeQueryConfig, queryWrapper, } from '..';
import { getConnectionConfig, getSubsocialApi } from './connection';
export function useSubsocialQuery(params, func, config, defaultConfig) {
    const mergedConfig = mergeQueryConfig(config, defaultConfig);
    return useQuery([params.key, params.data], queryWrapper(func, async () => {
        const api = await getSubsocialApi();
        return { api };
    }), mergedConfig);
}
export function useSubsocialMutation(wallet, transactionGenerator, config, defaultConfig) {
    const workerFunc = async (param) => {
        if (!wallet)
            throw new Error('You need to connect your wallet first!');
        const subsocialApi = await getSubsocialApi();
        return createTxAndSend(transactionGenerator, param, subsocialApi, { wallet, networkRpc: getConnectionConfig().substrateUrl }, config, defaultConfig);
    };
    return useMutation(workerFunc, {
        ...(defaultConfig || {}),
        ...config,
        onSuccess: makeCombinedCallback(defaultConfig, config, 'onSuccess'),
        onError: makeCombinedCallback(defaultConfig, config, 'onError'),
    });
}
