import { getSubstrateApi, SubsocialApi } from '@subsocial/api';
const DEFAULT_STAGING_CONFIG = {
    substrateUrl: 'wss://rco-para.subsocial.network',
    ipfsNodeUrl: 'https://staging-ipfs.subsocial.network',
    offchainUrl: 'https://staging-api.subsocial.network',
    useServer: {
        httpRequestMethod: 'get',
    },
};
const DEFAULT_PROD_CONFIG = {
    substrateUrl: 'wss://para.f3joule.space',
    offchainUrl: 'https://app.subsocial.network/offchain',
    ipfsNodeUrl: 'https://ipfs.subsocial.network',
};
let subsocialApi = null;
const presets = {
    staging: DEFAULT_STAGING_CONFIG,
    prod: DEFAULT_PROD_CONFIG,
};
const DEFAULT_CONFIG_PRESET = 'staging';
let config = presets[DEFAULT_CONFIG_PRESET];
export function getConnectionConfig() {
    return config;
}
export const setSubsocialConfig = (preset, customConfig) => {
    config = { ...presets[preset], ...customConfig };
};
export const getSubsocialApi = async () => {
    if (subsocialApi)
        return subsocialApi;
    const api = connectToSubsocialApi(config);
    subsocialApi = api;
    return subsocialApi;
};
async function connectToSubsocialApi(config) {
    const substrateApi = await getSubstrateApi(config.substrateUrl);
    return new SubsocialApi({ substrateApi, ...config });
}
